@@ -17,8 +17,8 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef MANAGEDEVICENAMES_H
-#define MANAGEDEVICENAMES_H
+#ifndef FAVORITES_H
+#define FAVORITES_H
 
 #include <QDialog>
 #include <QString>
@@ -39,117 +39,48 @@ class favorites : public QDialog
 {
 	Q_OBJECT
 public:
+	static constexpr const char * reverseModeOption = "-SiriKaliReverseMode" ;
+	static constexpr const char * volumeNeedNoPassword = "-SiriKaliVolumeNeedNoPassword" ;
+	static constexpr const char * mountReadOnly = "-SiriKaliMountReadOnly" ;
+
 	struct entry
 	{
-		entry()
+		class readOnly
 		{
-		}
-
-		entry( const QStringList& e )
-		{
-			this->config( e ) ;
-		}
-
-		entry( const QString& r )
-		{
-			this->config( r.split( '\t',QString::SkipEmptyParts ) ) ;
-		}
-
-		QStringList list( bool e = true ) const
-		{
-			if( e ){
-
-				return this->configString().split( '\t',QString::SkipEmptyParts ) ;
-			}else{
-				return { volumePath,
-					 mountPointPath,
-					 autoMountVolume,
-					 configFilePath,
-					 idleTimeOut,
-					 mountOptions } ;
-			}
-		}
-
-		QString string( char s = '\t' ) const
-		{
-			return this->list().join( QString( s ) ) ;
-		}
-
-		QString configString() const
-		{
-			auto _opt = [ ]( const QString& e )->QString{
-
-				if( e.isEmpty() ){
-
-					return "N/A" ;
-				}else{
-					return e ;
-				}
-			} ;
-
-			auto e = "%1\t%2\t%3\t%4\t%5\t%6\t" ;
-
-			return QString( e ).arg( volumePath,mountPointPath,autoMountVolume,
-						 _opt( configFilePath ),_opt( idleTimeOut ),
-						 _opt( mountOptions ) ) ;
-		}
-		QStringList configStringList() const
-		{
-			return this->configString().split( "\t",QString::SkipEmptyParts ) ;
-		}
-		bool operator!=( const favorites::entry& other ) const
-		{
-			return !( *this == other ) ;
-		}
-
-		bool operator==( const favorites::entry& other ) const
-		{
-			return  this->volumePath      == other.volumePath &&
-				this->mountPointPath  == other.mountPointPath &&
-				this->autoMountVolume == other.autoMountVolume &&
-				this->configFilePath  == other.configFilePath &&
-				this->idleTimeOut     == other.idleTimeOut &&
-				this->mountOptions    == other.mountOptions ;
-		}
-
-		bool autoMount() const
-		{
-			return autoMountVolume == "true" ;
-		}
+		public:
+			readOnly() ;
+			readOnly( bool e ) ;
+			operator bool() const ;
+			bool onlyRead() const ;
+			bool operator==( const readOnly& other ) ;
+		private:
+			bool m_readOnlyVolume = false ;
+			bool m_isSet = false ;
+		} ;
 
+		entry() ;
+		entry( const QStringList& e ) ;
+		entry( const QString& r ) ;
+		QStringList list( bool e = true ) const ;
+		QString string( char s = '\t' ) const ;
+		QString configString() const ;
+		QStringList configStringList() const;
+		bool operator!=( const favorites::entry& other ) const ;
+		bool operator==( const favorites::entry& other ) const ;
+		bool autoMount() const ;
+		QString sanitizedMountOptions() const ;
+		static QString sanitizedMountOptions( const QString& s ) ;
 		QString volumePath ;
 		QString mountPointPath ;
 		QString autoMountVolume ;
 		QString configFilePath ;
 		QString idleTimeOut ;
 		QString mountOptions ;
-
+		bool reverseMode = false ;
+		bool volumeNeedNoPassword = false ;
+		favorites::entry::readOnly readOnlyMode = readOnly() ;
 	private:
-		void config( const QStringList& e )
-		{
-			utility2::stringListToStrings( e,
-						      volumePath,
-						      mountPointPath,
-						      autoMountVolume,
-						      configFilePath,
-						      idleTimeOut,
-						      mountOptions ) ;
-
-			if( configFilePath == "N/A" ){
-
-				configFilePath.clear() ;
-			}
-
-			if( idleTimeOut == "N/A" ){
-
-				idleTimeOut.clear() ;
-			}
-
-			if( mountOptions == "N/A" ){
-
-				mountOptions.clear() ;
-			}
-		}
+		void config( const QStringList& e ) ;
 	};
 
 	enum class type{ sshfs,others } ;
@@ -187,8 +118,11 @@ private:
 	void addEntries( const QStringList& ) ;
 	Ui::favorites * m_ui ;
 	QWidget * m_parentWidget ;
+	favorites::type m_type ;
 	int m_editRow ;
 	bool m_reverseMode = false ;
+	bool m_volumeNeedNoPassword = false ;
+	bool m_mountReadOnly = false ;
 };
 
-#endif // MANAGEDEVICENAMES_H
+#endif // FAVORITES_H
