@@ -39,6 +39,8 @@
 #include "checkforupdates.h"
 #include "configoptions.h"
 #include "debugwindow.h"
+#include "settings.h"
+#include "systemsignalhandler.h"
 
 #include <vector>
 
@@ -87,6 +89,8 @@ private slots:
 	void createVolume( QAction * = nullptr ) ;
 	void slotMount( void ) ;
 	void unMountAll( void ) ;
+	void emergencyShutDown( void ) ;
+	std::function< void( systemSignalHandler::signal ) > getEmergencyShutDown() ;
 	void unMountAllAndQuit( void ) ;
 	void pbUmount( void ) ;
 	void slotTrayClicked( QSystemTrayIcon::ActivationReason = QSystemTrayIcon::Trigger ) ;
@@ -136,7 +140,7 @@ private:
 
 	secrets m_secrets ;
 
-	utility2::translator m_translator ;
+	settings::translator m_translator ;
 
 	QMenu * m_hidden_volume_menu = nullptr ;
 	QMenu * m_not_hidden_volume_menu = nullptr ;
@@ -153,6 +157,7 @@ private:
 	bool m_startHidden ;
 	bool m_autoOpenFolderOnMount ;
 	bool m_disableEnableAll = false ;
+	bool m_emergencyShuttingDown = false ;
 
 	QString m_sharedFolderPath ;
 	QString m_folderOpener ;
@@ -168,6 +173,8 @@ private:
 	configOptions m_configOptions ;
 
 	debugWindow m_debugWindow ;
+
+	systemSignalHandler m_signalHandler ;
 };
 
 #endif // MAINWINDOW_H
